// ==========================================================================
// $Id: multid_init.cpp,v 1.2 2014/09/13 19:48:03 jlang Exp $
// CSI2372 example Code for lecture 3
// ==========================================================================
// (C)opyright:
//
//   Jochen Lang
//   SITE, University of Ottawa
//   800 King Edward Ave.
//   Ottawa, On., K1N 6N5
//   Canada. 
//   http://www.site.uottawa.ca
// 
// Creator: jlang (Jochen Lang)
// Email:   jlang@site.uottawa.ca
// ==========================================================================
// $Log: multid_init.cpp,v $
// Revision 1.2  2014/09/13 19:48:03  jlang
// Added C++11 features to lecture 3
//
// Revision 1.1  2006/09/11 21:38:36  jlang
// Check-in for lecture 3
//
//
// ==========================================================================
#include <iostream>
#include <iomanip>

using std::cout;
using std::endl;
using std::setw;

const int numRows = 3, numCols = 4;


/** Print a 2D array with pre-determined number of columns 
 *  (numCols must be defined at compile time!)
 *  Array notation is used!
 *  The array is passed by pointer to rows of length numCol */
void printIntArray2D_array( int numbers[][numCols], int numRows ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r][c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}

/** Print a 2D array with pre-determined number of columns 
 *  (numCols must be defined at compile time!)
 *  Pointer notation is used!
 *  The array is passed by pointer to rows of length numCol */
void printIntArray2D_ptr( int (*numbers)[numCols], int numRows ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r][c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}


/** Print a 2D array by passing a pointer to type
 *  Memory layout must be contiguous. */
void printIntArray2D_ptrType( int *numbers, 
			      const int numRows, const int numCols ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r*numCols+c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}


int main() {
  // numRows and numCols have to be defined at compile time
  int numbersA[numRows][numCols]; // not initialized
  printIntArray2D_array( numbersA, numRows );

  // numRows is redundant (inferred from number of elements)
  // nested brackets are redundant but increase readability
  int numbersB[numRows][numCols]{{ 0, 1, 2, 3 },
				    { 4, 5, 6, 7 },
				    { 8, 9, 10, 11 }}; 
  printIntArray2D_ptr( numbersB, numRows );
  int numbersC[numRows][numCols]
    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
  printIntArray2D_array( numbersC, numRows ); 

  // Pass the array as a pointer to the first element
  // and hope for contiguous memory layout
  int numbersD[][numCols]
    { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
  printIntArray2D_ptrType(&numbersD[0][0], numRows, numCols ); 
  
  return 0;
}
