// ==========================================================================
// $Id: semi_dynamic2d.cpp,v 1.1 2006/09/11 21:38:36 jlang Exp $
// CSI2372 example Code for lecture 3
// ==========================================================================
// (C)opyright:
//
//   Jochen Lang
//   SITE, University of Ottawa
//   800 King Edward Ave.
//   Ottawa, On., K1N 6N5
//   Canada. 
//   http://www.site.uottawa.ca
// 
// Creator: jlang (Jochen Lang)
// Email:   jlang@site.uottawa.ca
// ==========================================================================
// $Log: semi_dynamic2d.cpp,v $
// Revision 1.1  2006/09/11 21:38:36  jlang
// Check-in for lecture 3
//
//
// ==========================================================================
#include <iostream>
#include <iomanip>

using std::cout;
using std::endl;
using std::setw;

// All dimensions except first must be determined at compile time
const int numCols = 4;

/** Print a 2D array with pre-determined number of columns 
 *  (numCols must be defined at compile time!)
 *  Array notation is used!
 *  The array is passed by pointer to rows of length numCol */
void printIntArray2D_array( int numbers[][numCols], int numRows ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r][c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}


/** Print a 2D array with pre-determined number of columns 
 *  (numCols must be defined at compile time!)
 *  Pointer notation is used!
 *  The array is passed by pointer to rows of length numCol */
void printIntArray2D_ptr( int (*numbers)[numCols], int numRows ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r][c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}

/** Print a 2D array by passing a pointer to type
 *  Memory layout must be contiguous. */
void printIntArray2D_ptrType( int *numbers, 
			      const int numRows, const int numCols ) {
  for (int r=0;r<numRows;r++) {
    for (int c=0;c<numCols;c++) {
      cout << setw(10) << numbers[r*numCols+c] << '\t';
    }
    cout << endl;
  }
  cout << endl;
}

int main() {
  int numRows = 3; // first dimension can be determined at run-time
  // numCols must be known at compile time!
  int (*numbers)[numCols] = new int[numRows][numCols]; 
  // Fill array with a sequence of numbers
  for(int r=0; r<numRows; r++) {
    for(int c=0; c<numCols; c++) {
      numbers[r][c] = r*numCols + c;
    }
  } 
  // print the array by array notation
  printIntArray2D_array( numbers, numRows ); 
  // print the array by ptr notation
  printIntArray2D_ptr( numbers, numRows ); 
  // Pass the array as a pointer to the first element
  // and hope for contiguous memory layout
  printIntArray2D_ptrType(&numbers[0][0], numRows, numCols ); 
  
  // delete the array
  delete [] numbers;
  return 0;
}
